package java.net;

import java.io.ObjectStreamException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.misc.Unsafe;
import java.io.ObjectStreamField;

public class InetSocketAddress extends SocketAddress
{
    private final transient InetSocketAddress.InetSocketAddressHolder holder;
    private static final long serialVersionUID = 5076001401234631237L;
    private static final ObjectStreamField[] serialPersistentFields;
    private static final long FIELDS_OFFSET;
    private static final Unsafe UNSAFE;
    
    private static int checkPort(int i) {
        if (i < 0 || i > 65535) {
            throw new IllegalArgumentException("port out of range:" + i);
        }
        return i;
    }
    
    private static String checkHost(String s) {
        if (s == null) {
            throw new IllegalArgumentException("hostname can't be null");
        }
        return s;
    }
    
    public InetSocketAddress(int n) {
        this(InetAddress.anyLocalAddress(), n);
    }
    
    public InetSocketAddress(InetAddress inetAddress, int n) {
        this.holder = new InetSocketAddress.InetSocketAddressHolder(null, (inetAddress == null) ? InetAddress.anyLocalAddress() : inetAddress, checkPort(n));
    }
    
    public InetSocketAddress(String s, int n) {
        checkHost(s);
        InetAddress byName = null;
        String s2 = null;
        try {
            byName = InetAddress.getByName(s);
        }
        catch (UnknownHostException ex) {
            s2 = s;
        }
        this.holder = new InetSocketAddress.InetSocketAddressHolder(s2, byName, checkPort(n));
    }
    
    private InetSocketAddress(int n, String s) {
        this.holder = new InetSocketAddress.InetSocketAddressHolder(s, null, n);
    }
    
    public static InetSocketAddress createUnresolved(String s, int n) {
        return new InetSocketAddress(checkPort(n), checkHost(s));
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("hostname", this.holder.hostname);
        putFields.put("addr", this.holder.addr);
        putFields.put("port", this.holder.port);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        String s = (String)fields.get("hostname", null);
        InetAddress inetAddress = (InetAddress)fields.get("addr", null);
        int value = fields.get("port", -1);
        checkPort(value);
        if (s == null && inetAddress == null) {
            throw new InvalidObjectException("hostname and addr can't both be null");
        }
        InetSocketAddress.UNSAFE.putObject(this, InetSocketAddress.FIELDS_OFFSET, new InetSocketAddress.InetSocketAddressHolder(s, inetAddress, value));
        return;
    }
    
    private void readObjectNoData() throws ObjectStreamException {
        throw new InvalidObjectException("Stream data required");
    }
    
    public final int getPort() {
        return InetSocketAddress.InetSocketAddressHolder.access$400(this.holder);
    }
    
    public final InetAddress getAddress() {
        return InetSocketAddress.InetSocketAddressHolder.access$500(this.holder);
    }
    
    public final String getHostName() {
        return InetSocketAddress.InetSocketAddressHolder.access$600(this.holder);
    }
    
    public final String getHostString() {
        return InetSocketAddress.InetSocketAddressHolder.access$700(this.holder);
    }
    
    public final boolean isUnresolved() {
        return InetSocketAddress.InetSocketAddressHolder.access$800(this.holder);
    }
    
    @Override
    public String toString() {
        return this.holder.toString();
    }
    
    @Override
    public final boolean equals(Object o) {
        return o != null && o instanceof InetSocketAddress && this.holder.equals(((InetSocketAddress)o).holder);
    }
    
    @Override
    public final int hashCode() {
        return this.holder.hashCode();
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("hostname", String.class), new ObjectStreamField("addr", InetAddress.class), new ObjectStreamField("port", Integer.TYPE) };
        try {
            Unsafe unsafe = Unsafe.getUnsafe();
            FIELDS_OFFSET = unsafe.objectFieldOffset(InetSocketAddress.class.getDeclaredField("holder"));
            UNSAFE = unsafe;
        }
        catch (ReflectiveOperationException cause) {
            throw new Error(cause);
        }
    }
    
    private static class InetSocketAddressHolder
    {
        private String hostname;
        private InetAddress addr;
        private int port;
        
        private InetSocketAddressHolder(String hostname, InetAddress addr, int port) {
            this.hostname = hostname;
            this.addr = addr;
            this.port = port;
        }
        
        private int getPort() {
            return this.port;
        }
        
        private InetAddress getAddress() {
            return this.addr;
        }
        
        private String getHostName() {
            if (this.hostname != null) {
                return this.hostname;
            }
            if (this.addr != null) {
                return this.addr.getHostName();
            }
            return null;
        }
        
        private String getHostString() {
            if (this.hostname != null) {
                return this.hostname;
            }
            if (this.addr != null) {
                if (this.addr.holder().getHostName() != null) {
                    return this.addr.holder().getHostName();
                }
                return this.addr.getHostAddress();
            }
            else {
                return null;
            }
        }
        
        private boolean isUnresolved() {
            return this.addr == null;
        }
        
        @Override
        public String toString() {
            if (this.isUnresolved()) {
                return this.hostname + ":" + this.port;
            }
            return this.addr.toString() + ":" + this.port;
        }
        
        @Override
        public final boolean equals(Object o) {
            if (o == null || !(o instanceof InetSocketAddress.InetSocketAddressHolder)) {
                return false;
            }
            InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder = (InetSocketAddress.InetSocketAddressHolder)o;
            int equals;
            if (this.addr != null) {
                equals = this.addr.equals(inetSocketAddressHolder.addr);
            }
            else if (this.hostname != null) {
                equals = (inetSocketAddressHolder.addr == null && this.hostname.equalsIgnoreCase(inetSocketAddressHolder.hostname));
            }
            else {
                equals = (inetSocketAddressHolder.addr == null && inetSocketAddressHolder.hostname == null);
            }
            return equals && this.port == inetSocketAddressHolder.port;
        }
        
        @Override
        public final int hashCode() {
            if (this.addr != null) {
                return this.addr.hashCode() + this.port;
            }
            if (this.hostname != null) {
                return this.hostname.toLowerCase().hashCode() + this.port;
            }
            return this.port;
        }
        
        static int access$400(InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder) {
            return inetSocketAddressHolder.getPort();
        }
        
        static InetAddress access$500(InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder) {
            return inetSocketAddressHolder.getAddress();
        }
        
        static String access$600(InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder) {
            return inetSocketAddressHolder.getHostName();
        }
        
        static String access$700(InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder) {
            return inetSocketAddressHolder.getHostString();
        }
        
        static boolean access$800(InetSocketAddress.InetSocketAddressHolder inetSocketAddressHolder) {
            return inetSocketAddressHolder.isUnresolved();
        }
    }
}
